@@ -1,16 +1,16 @@
 """
 Base for UI generation
 """
+import streamlit as st
 from collections import ChainMap
 from typing import Callable, Any, Union, Mapping, Iterable
 from functools import partial
 import typing
 from warnings import warn
-
-import streamlit as st
-
 from i2 import Sig
-from streamlitfront.app_maker import AppMaker
+from front import AppMaker
+
+from streamlitfront.spec_maker import SpecMaker
 from streamlitfront.session_state import get_state, _SessionState
 from streamlitfront.util import func_name, build_element_factory
 
@@ -352,17 +352,18 @@ def mk_app(objs: Iterable, config: Map = None, convention: Map = None):
 
     >>> app = mk_app(funcs)
 
-    The default configuration for the application is define by the convention object:
-    ``dflt_convention``. But you can overwrite parts or the entire configuration by
-    setting the ``config`` parameter. The configuration is composed of three parts:
-    app, obj and rendering.
+    The default configuration for the application is defined in
+    ``streamlitfront/spec_maker.py`` and ``front/spec_maker_base.py``. But you can
+    overwrite parts or the entire configuration by setting the ``config`` parameter.
+    The configuration is composed of three parts: app, obj and rendering.
 
     The app configuration:
     By default, the application name is "My Front Application", but you can set the
     title of the application as follow:
 
+    >>> from front import APP_KEY
     >>> config = {
-    ...     'app': {
+    ...     APP_KEY: {
     ...         'title': 'Another application name'
     ...     }
     ... }
@@ -372,11 +373,12 @@ def mk_app(objs: Iterable, config: Map = None, convention: Map = None):
     You can define a wrapper to transform the initial object into an output of your
     choice to be rendered:
 
-    >>> from front.app_maker_base import dflt_trans
+    >>> from front.util import dflt_trans
+    >>> from front import OBJ_KEY
     >>> def trans(objs: Iterable):
     ...     return dflt_trans(reversed(objs))
     >>> config = {
-    ...     'obj': {
+    ...     OBJ_KEY: {
     ...         'trans': trans
     ...     }
     ... }
@@ -387,14 +389,17 @@ def mk_app(objs: Iterable, config: Map = None, convention: Map = None):
     For instance, you can choose to render a text input instead of a number input for a
     specific parameter of a specific function:
 
-    >>> from front.elements import INT_INPUT_SLIDER_COMPONENT
+    >>> from front import ELEMENT_KEY, RENDERING_KEY
+    >>> from streamlitfront.elements import IntSliderInput
     >>> config = {
-    ...     'rendering': {
-    ...         'Foo': {
-    ...             'inputs': {
-    ...                 'a': {
-    ...                     'component': INT_INPUT_SLIDER_COMPONENT,
-    ...                     'max_value': 10
+    ...     RENDERING_KEY: {
+    ...         'foo': {
+    ...             'execution': {
+    ...                 'inputs': {
+    ...                     'a': {
+    ...                         ELEMENT_KEY: IntSliderInput,
+    ...                         'max_value': 10
+    ...                     }
     ...                 }
     ...             }
     ...         }
@@ -405,18 +410,20 @@ def mk_app(objs: Iterable, config: Map = None, convention: Map = None):
     Obviously, you can combine the three types of configuration:
 
     >>> config = {
-    ...     'app': {
+    ...     APP_KEY: {
     ...         'title': 'Another application name'
     ...     },
-    ...     'obj': {
+    ...     OBJ_KEY: {
     ...         'trans': trans
     ...     },
-    ...     'rendering': {
-    ...         'Foo': {
-    ...             'inputs': {
-    ...                 'a': {
-    ...                     'component': INT_INPUT_SLIDER_COMPONENT,
-    ...                     'max_value': 10
+    ...     RENDERING_KEY: {
+    ...         'foo': {
+    ...             'execution': {
+    ...                 'inputs': {
+    ...                     'a': {
+    ...                         ELEMENT_KEY: IntSliderInput,
+    ...                         'max_value': 10
+    ...                     }
     ...                 }
     ...             }
     ...         }
@@ -429,51 +436,110 @@ def mk_app(objs: Iterable, config: Map = None, convention: Map = None):
     make sure that all configuations are defined. Otherwise, the application would
     crash or behave unexpectedly.
 
-    >>> from meshed import DAG
-    >>> from front.elements import (
-    ...     VIEW_CONTAINER,
-    ...     FLOAT_INPUT_SLIDER_COMPONENT,
-    ...     TEXT_INPUT_COMPONENT,
-    ...     EXEC_SECTION_CONTAINER,
-    ...     SECTION_CONTAINER,
-    ...     GRAPH_COMPONENT
+    >>> from collections.abc import Callable
+    >>> from front import NAME_KEY, DEFAULT_INPUT_KEY
+    >>> from streamlitfront.elements import (
+    ...     App,
+    ...     ExecSection,
+    ...     FloatSliderInput,
+    ...     IntInput,
+    ...     TextInput,
+    ...     TextOutput,
+    ...     TextSection,
+    ...     View,
     ... )
+    >>> from streamlitfront.spec_maker import get_stored_value
     >>> 
     >>> convention = {
-    ...     'app': {
+    ...     APP_KEY: {
     ...         'title': 'Another application name'
     ...     },
-    ...     'obj': {
+    ...     OBJ_KEY: {
     ...         'trans': trans
     ...     },
-    ...     'rendering': {
-    ...         DAG: {
-    ...             'container': VIEW_CONTAINER,
+    ...     RENDERING_KEY: {
+    ...         ELEMENT_KEY: App,
+    ...         Callable: {
+    ...             ELEMENT_KEY: View,
+    ...             'description': {
+    ...                 ELEMENT_KEY: TextSection,
+    ...                 NAME_KEY: 'Description',
+    ...                 'content': lambda o: o.__doc__,
+    ...             },
     ...             'execution': {
-    ...                 'container': EXEC_SECTION_CONTAINER,
-    ...                 'name': 'Execution',
+    ...                 ELEMENT_KEY: ExecSection,
+    ...                 NAME_KEY: 'Execution',
+    ...                 'stored_value_getter': get_stored_value,
     ...                 'inputs': {
+    ...                     int: {ELEMENT_KEY: IntInput,},
     ...                     float: {
-    ...                         'component': FLOAT_INPUT_SLIDER_COMPONENT,
+    ...                         ELEMENT_KEY: FloatSliderInput,
     ...                         'format': '%.2f',
     ...                         'step': 0.01,
     ...                     },
-    ...                     Any: {'component': TEXT_INPUT_COMPONENT,},
+    ...                     Any: {ELEMENT_KEY: TextInput,},
+    ...                     DEFAULT_INPUT_KEY: {NAME_KEY: lambda p: p.name}
     ...                 },
-    ...             },
-    ...             'graph': {
-    ...                 'container': SECTION_CONTAINER,
-    ...                 'name': 'Flow',
-    ...                 'component': GRAPH_COMPONENT,
-    ...                 'display': True,
-    ...                 'display_for_single_node': False,
+    ...                 'output': {
+    ...                     ELEMENT_KEY: TextOutput,
+    ...                     NAME_KEY: 'Output',
+    ...                 }
     ...             }
-    ...         },
-    ...     },
+    ...         }
+    ...     }
     ... }
     >>> app = mk_app(funcs, convention=convention)
 
-    :param objs: The target objects to render in the streamlit application.
+    You can also add new elements to the GUI by defining it first then add it to the
+    configuration. In the following example, we display the dot graph of a DAG using
+    our own implementation of the component. It is then referenced in the configuration
+    with the parameters to intantiate it.
+
+    >>> from meshed import DAG
+    >>> from front.types import FrontElementName
+    >>> from front.elements import FrontComponentBase
+    >>> 
+    >>> def b(a: int):
+    ...     return 2 ** a
+    >>> def d(c: int):
+    ...     return 10 - (5 ** c)
+    >>> def result(b, d):
+    ...     return b * d
+    >>> dag = DAG((b, d, result))
+    >>> 
+    >>> class Graph(FrontComponentBase):
+    ...     def __init__(
+    ...         self,
+    ...         obj: DAG,
+    ...         name: FrontElementName = None,
+    ...         use_container_width: bool = False
+    ...     ):
+    ...         super().__init__(obj=obj, name=name)
+    ...         self.use_container_width = use_container_width
+    ... 
+    ...     def render(self):
+    ...         with st.expander(self.name, True):
+    ...             dag: DAG = self.obj
+    ...             st.graphviz_chart(
+    ...                 figure_or_dot=dag.dot_digraph(),
+    ...                 use_container_width=self.use_container_width
+    ...             )
+    >>> 
+    >>> config = {
+    ...     APP_KEY: {'title': 'DAG App'},
+    ...     RENDERING_KEY: {
+    ...         DAG: {
+    ...             'graph': {
+    ...                 ELEMENT_KEY: Graph,
+    ...                 NAME_KEY: 'Flow',
+    ...                 'use_container_width': True
+    ...             }
+    ...         }
+    ...     }
+    ... }
+    >>> app = mk_app([dag], config=config)
+
+    :param objs: The target objects to crender in the streamlit application.
     :type objs: Iterable
     :param config: The configuration object for the application. Overwrites the
     convention for every value present in the configuration object. See above for more
@@ -483,5 +549,5 @@ def mk_app(objs: Iterable, config: Map = None, convention: Map = None):
     configuration. See above for more details.
     :type objs: Map
     """
-    app_maker = AppMaker()
+    app_maker = AppMaker(spec_maker_factory=SpecMaker)
     return app_maker.mk_app(objs, config, convention)
