@@ -10,20 +10,20 @@ def foo(a: int = 0, b: int = 0, c=0):
 
 
 class Klass:
-    def __init__(self, greeting="hello", confuser_x: float = 3.14):
+    def __init__(self, greeting='hello', confuser_x: float = 3.14):
         self.greeting = greeting
         self.confuser_x = confuser_x
 
     def bar(self, x):
         """bar greets its input"""
-        return f"{self.greeting} {x}"
+        return f'{self.greeting} {x}'
 
     # def konfuser(self, a: int = 0):
     #     return (a ** 2) * self.confuser_x
 
 
 def i_am_confused(self, a: int = 0):
-    return (a**2) * self.confuser_x
+    return (a ** 2) * self.confuser_x
 
 
 klass_instance = Klass()
@@ -34,9 +34,9 @@ klass_instance.konfuser = MethodType(i_am_confused, klass_instance)
 funcs = [foo, klass_instance.bar, klass_instance.konfuser]
 
 
-if __name__ == "__main__":
+if __name__ == '__main__':
     from streamlitfront.base import dispatch_funcs
 
-    print("file: {}".format(os.path.realpath(__file__)))
+    print('file: {}'.format(os.path.realpath(__file__)))
     app = dispatch_funcs(funcs)
     app()
